package com.localmatters.mobile.horoscope;

import com.localmatters.mobile.horoscope.mag.HoroscopeListing;
import com.localmatters.mobile.horoscope.mag.HoroscopeService;
import com.localmatters.mobile.horoscope.mag.HoroscopeServiceSoap;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.WebServiceRef;

/**
 * The horoscope EJB.
 *
 * @author Daniel Cavalcanti
 */
@WebService()
@Stateless()
public class HoroscopeBean implements HoroscopeRemote, HoroscopeLocal {
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
    /**
     * The MAG horoscope web service.
     */
    @WebServiceRef(wsdlLocation = "http://webservices.myareaguide.com/horoscopes/Horoscopes.asmx?WSDL")
    private com.localmatters.mobile.horoscope.mag.HoroscopeService service;
    
    /**
     * The MAG horoscope web service port.
     */
    private HoroscopeServiceSoap port;
    
//    /**
//     * The QName namespace URI.
//     */
//    @Resource(name = "qname-namespace-uri")
//    private String namespaceURI;
//    
//    /**
//     * The QName local part.
//     */
//    @Resource(name = "qname-local-part")
//    private String localPart;
//    
//    /**
//     * The MAG-WS WSDL URL.
//     */
//    @Resource(name = "wsdl-url")
//    private String wsdlURL;
//    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of HoroscopeBean.
     */
    public HoroscopeBean() {
    }
    
    /**
     * Establishes connection to MAG-WS.
     */
    @PostConstruct()
    private void init() {
        
        port = service.getHoroscopeServiceSoap();
        
//        try {
//            HoroscopeService service = new HoroscopeService(new URL(wsdlURL), new QName(namespaceURI, localPart));
//            port = service.getHoroscopeServiceSoap();
//        } catch (MalformedURLException ex) {
//            System.err.println("Error: " + ex.getMessage());
//            ex.printStackTrace();
//        }
//        
    }
    
    // **********************************************************************
    // Horoscope methods
    // **********************************************************************
    
    @WebMethod()
    public HoroscopeDescription horoscope() {
        
        // the horoscope listing
        HoroscopeListing listing = port.getHoroscopes();
        
        // encapsulate listing into horoscope description bean
        List<SignDescription> signDescriptions = getSignDescriptions(listing);
        
        // return horoscope description bean
        return new HoroscopeDescription(listing.getDate(), signDescriptions);
        
    }
    
    @WebMethod()
    public HoroscopeDescription horoscopeByDate(@WebParam() XMLGregorianCalendar date) {
        
        // the horoscope listing
        HoroscopeListing listing = port.getHoroscopesByDate(date);
        
        // encapsulate listing into horoscope description bean
        List<SignDescription> signDescriptions = getSignDescriptions(listing);
        
        // return horoscope description bean
        return new HoroscopeDescription(listing.getDate(), signDescriptions);
        
    }
    
    // **********************************************************************
    // Auxiliary methods
    // **********************************************************************
    
    private List<SignDescription> getSignDescriptions(HoroscopeListing listing) {
        
        // create sign descriptions list
        List<SignDescription> signDescriptions = new ArrayList<SignDescription>(listing.getHoroscope().size());
        
        // populate sign descriptions list
        for (com.localmatters.mobile.horoscope.mag.Horoscope horoscope : listing.getHoroscope()) {
            
            SignDescription signDescription = new SignDescription();
            
            signDescription.setSign(horoscope.getSign());
            signDescription.setCompatibleSign(horoscope.getCompatibleSign());
            signDescription.setFinancialOutlook(horoscope.getFinancialOutlook());
            signDescription.setText(horoscope.getText());
            signDescription.setFortunateAstrologicalNumber(horoscope.getFortunateAstrologicalNumber());
            
            signDescriptions.add(signDescription);
            
        }
        
        // return sign descriptions list
        return signDescriptions;
        
    }
    
}
